package ftp

import (
	"crypto/tls"
	"net"
	"testing"
	"time"

	osm "git.loafle.net/overflow/service_matcher-go"
)

func TestFTP(t *testing.T) {

	m := NewMatcher()

	conn, err := net.Dial("tcp", "192.168.1.229:21")

	if err != nil {
		t.Error(err)
	}
	defer conn.Close()

	matchCtx := osm.NewMatchCtx("192.168.1.229", 21)

	for i := 0; i < m.PacketCount(matchCtx); i++ {
		_, err := conn.Write(m.Packet(matchCtx, i).Buffer)
		if err != nil {
			t.Error(err)
		}
		bytes := make([]byte, 1024)
		n, _ := conn.Read(bytes)
		p := osm.NewPacket(bytes, n)

		if err := m.Match(nil, i, p); err != nil {
			t.Error(err)
			return
		}
	}
	t.Log(m.Name(matchCtx))
	t.Log(matchCtx)
}

func TestFTPS(t *testing.T) {

	m := NewMatcher()

	dialer := &net.Dialer{
		Timeout: 5 * time.Second,
	}

	conn, err := tls.DialWithDialer(
		dialer,
		"tcp",
		"192.168.1.201:990",
		&tls.Config{
			InsecureSkipVerify: true,
			ServerName:         "192.168.1.201",
		},
	)

	if err != nil {
		t.Error(err)
	}
	defer conn.Close()

	matchCtx := osm.NewMatchCtx("192.168.1.201", 990)

	for i := 0; i < m.PacketCount(matchCtx); i++ {
		_, err := conn.Write(m.Packet(matchCtx, i).Buffer)
		if err != nil {
			t.Error(err)
		}
		bytes := make([]byte, 1024)
		n, _ := conn.Read(bytes)
		p := osm.NewPacket(bytes, n)

		if err := m.Match(nil, i, p); err != nil {
			t.Error(err)
		}
	}
	t.Log(m.Name(matchCtx))
	t.Log(matchCtx)
}
